---@param relation ReplaceRelation
---@param world World
local function applyReplacementRelation(relation, world)
    for k, v in pairs(relation.replacement) do
        relation.entityToModify[k] = v
    end
    world:addEntity(relation.entityToModify)
end

menuController = filteredSystem(
    "menuController",
    { menuItems = Arr(T.Selectable), inputState = T.InputState },
    function(e, _, system)
        for _, menuItem in pairs(e.menuItems) do
            if menuItem.highlighted then
                if e.inputState.aJustPressed then
                    -- Prepare to remove the menu and all menu items
                    system.world:removeEntity(e)
                    for _, item in pairs(e.menuItems) do
                        system.world:removeEntity(item)
                    end

                    -- TODO: Larger menu that stays open for more purchases before the next round
                    world:addEntity({ roundAction = "start" })

                    if menuItem.replacements then
                        for _, v in ipairs(menuItem.replacements) do
                            applyReplacementRelation(v, system.world)
                        end
                    end
                    if menuItem.additions then
                        for _, v in ipairs(menuItem.additions) do
                            system.world:addEntity(v)
                        end
                    end
                end

                if e.inputState.downJustPressed and menuItem.navigateDown then
                    menuItem.highlighted = false
                    menuItem.navigateDown.highlighted = true
                    return
                end

                if e.inputState.upJustPressed and menuItem.navigateUp then
                    menuItem.highlighted = false
                    menuItem.navigateUp.highlighted = true
                    return
                end
            end
        end

        for _, menuItem in pairs(e.menuItems) do
            if menuItem.highlighted then
                menuItem.drawAsText.style = TextStyle.Inverted
            else
                menuItem.drawAsText.style = TextStyle.Bordered
            end
        end
    end
)
